==> ldic-0.1.2.tar.gz: ldic-0.1.2/src/ldic.c <==
/* L-Dic(Linux Dictionary)
 * Copyright (C) 2002 Baek SeungSoon.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <malloc.h>
#include <string.h>
#include <getopt.h>

#include "ldic.h"
#include "wlist.h"
#include "rcfile.h"
#include "readdic.h"
#include "tagtrans.h"

/* global
 */
struct dic_t dic;
int cmdOpt = 0;		/* Ӱ˻ ɼ */
int quickOpt = 0;	/* ˻ ɼ */
int phonOpt = 0;	/* ȣ  */
int dbOpt = 0;		/*   */
int origOpt = 0;	/* tag ž  */

/* dicLoad:   б
 */
FILE
*dicLoad(char *dicName)
{
	FILE *fp;
	
	fp = fopen(dicName, "r");
	if (fp == NULL) {
		fprintf(stderr, "Error: %s, %s\n", dicName, getErrMsg("dic"));
		return NULL;
	}

	if (isDic(fp) == 0) {
		fprintf(stderr, "Error: %s, %s\n", dicName, getErrMsg("notdic"));
		return NULL;
	}
	
	dic = readHeader(fp);

	return fp;
}

/* dicSearch:   ܾ ˻
 */
int
dicSearch(char *dicName, char *word)
{
	char *s;
	long w_offset;
	
	FILE *fp;
	
	fp = dicLoad(dicName);
	if (fp == NULL)
		return 0;

	printf("%s ˻..\n", dicName);
	w_offset = FileSearchWord(dic.l_offset_s, dic.l_offset_e, word, fp);
	
	if (w_offset == -1) {
		fclose(fp);
		return 0;
	}
	
	s = dbSearchMean(dic.m_offset_s, w_offset, fp);
	if (phonOpt == 1) {
		if (dic.font == 1) {
			printPhon(s, fp);
		} else {
			fprintf(stderr, "Error: %s\n", getErrMsg("phon"));
		}
	} else {
		printMean(s);
	}
	
	fclose(fp);
	return 1;
}

/* allSearch:   ܾ ˻
 */
void
allSearch(char *word)
{
	int i;
	int success = 0;
	
	for (i = 0; i < DICTIONARY_MAX; i++) {
		if (dic_list[i] == NULL) {
			success = 0;
			continue;
		}
		
		success = dicSearch(dic_list[i], word);

		if (success)
			break;
	}
	
	if (success == 0) {
		fprintf(stderr, "Error: %s\n", getErrMsg("word"));
	}
}

/* commandLine: ܾ  ˻
 */
void
commandLine(void)
{
	int i;
	WList *wlist;
	char word[BUFSIZ+1];
	FILE *fp;
	
	write(1, "Words List Loding...\t\t", 23);
	for (i = 0; i < DICTIONARY_MAX; i++) {
		if (dic_list[i] == NULL) {
			continue;
		}
		
		fp = dicLoad(dic_list[i]);
		if (fp == NULL) {
			continue;
		}
		
		wlist = memoryList(i, dic.l_offset_s, dic.l_offset_e, fp);
		fclose(fp);
	}
	printf("[Done]\n");

	for (;;) {
		WList *temp;
		
		write(1, "Search Word ('/exit'): ", 23);
		
		fgets(word, sizeof(word), stdin);
		word[strlen(word)-1] = '\0';

		if (word[0] == '/') {
			if(strcmp(word, "/exit") == '\0') {
				break;
			} else {
				printf("Error: %s\n", getErrMsg("cmd"));
			}
			continue;
		}

		temp = ListSearchWord(wlist, word);
		if (temp != NULL) {
			char *s;
			fp = dicLoad(dic_list[temp->num]);
			s = dbSearchMean(dic.m_offset_s, temp->offset, fp);
			printMean(s);
			fclose(fp);
		} else {
			printf("Error: %s\n", getErrMsg("word"));
		}
	}

	freeall(wlist);
}

/* printPhon: ܾ ȣ 
 */
void
printPhon(char *mean, FILE *fp)
{
	int i;
	int x, y;
	int s_x, s_y;
	char **phonlist;
	struct fnt_t fnt;

	char screen[200][80];

	phonlist = tagList(mean, "img src");
	printf("[_phonetic symbol_]\n");
	s_x = s_y = 0;
	bzero(&screen, sizeof(screen));
	for (i = 0; phonlist[i] != NULL; i++) {
		char *s;
		s = strrchr(phonlist[i], '.');
		*s = '\0';
		s = strrchr(phonlist[i], '/');
		s++;
		
		if (s_x > 79) {
			s_x = 0;
			s_y += 20;
		}

		if (s_y > 200) {
			break;
		}

		fnt = readFnt(dic.f_offset_s, dic.f_offset_e, s, fp);
		if (fnt.img == NULL)
			continue;
		
		for (y = 0; y < fnt.y; y++) {
			for (x = 0; x < fnt.x; x++) {
				screen[y+s_y][x+s_x] = fnt.img[(fnt.x*y)+x];
			}
		}
		
		s_x += fnt.x;
	}

	for (y = 0; y < s_y+20; y++) {
		for (x = 0; x < 80; x++) {
			printf("%c", screen[y][x]);
		}
		printf("\n");
	}
}

/* printMean: ܾ  
 */
void
printMean(char *s)
{
	if (origOpt == 1) {
		printf("[_orig mean_]\n%s\n", s);
	} else {
		char *mean;
		mean = tagRemove(s);
		printf("[_mean_]\n%s\n", mean);
	}
}

/* getErrMsg: err شϴ ޼ ȯ
 */
char
*getErrMsg(char *err)
{
	static char retval[BUFSIZ+1];
	
	char *errList[] = {
		"arg",    "Էµ ܾ ϴ.",
		"list",   " ã  ϴ.",
		"dic",    " ã  ϴ.",
		"notdic", "   Դϴ.",
		"word",   "ܾ ã  ϴ.",
		"phon",   "ȣ  ϴ.",
		"cmd",    "ӵ  ƴմϴ.",
		NULL
	};

	int i;

	for (i = 0; errList[i] != NULL; i += 2) {
		if (strcmp(err, errList[i]) == '\0') {
			strcpy(retval, errList[i+1]);
			break;
		}
	}

	if (errList[i] == NULL) {
		strcpy(retval, "    ߻߽ϴ.");
	}

	return retval;
}

/* usage: α׷  
 */
void
usage(const char *str)
{
	char *s;
	
	if ((s = strrchr(str, '/')) != NULL) {
		++s;
	} else {
		s = strdup(str);
	}

	printf(": %s [ɼ]... [ܾ]\n", s);
	printf("\n");
	printf("  -c,     \tӰ˻\n");
	printf("  -q WORD,\t˻  \n");
	printf("  -p WORD,\tȣ \n");
	printf("  -o,\n");
	printf("  -o WORD,\ttag ž \n");
	printf("  -d FILE,\t \n");
	printf("  -h,     \t   \n");
	printf("  -v,     \tα׷    \n");
	printf("\n");
	printf("<tadboy@ldic.kldp.net>    ּ\n");
	
	exit(0);
}

/* version: α׷  
 */
void
version(void)
{
	printf("L-Dic(Linux Dictionary) Version %s\n", VERSION);
	printf("¼  ϴ.\n\n");
	printf("Copyright (C) 2002 Free Software Foundation, Inc.\n");
	printf(" α׷  ƮԴϴ. "
		"  ҽ Ͻʽÿ. "
		"ǰ̳ Ư   ռ Ͽ, "
		"   ʽϴ.\n");
}

/* main: α׷ ϰ, ɼ ó
 */
int
main(int argc, char *argv[])
{
	char c;
	char word[BUFSIZ+1];
	char *dicName = NULL;

	while ((c =  getopt(argc, argv,	"cqpod:h?v")) != -1) {
		switch (c) {
			case 'c':
				cmdOpt = 1;
				break;
			case 'q':
				quickOpt = 1;
				break;
			case 'p':
				phonOpt = 1;
				break;
			case 'o':
				origOpt = 1;
				break;
			case 'd':
				dbOpt = 1;
				dicName = strdup(optarg);
				break;
			case 'h':
			case '?':
				usage(argv[0]);
				break;
			case 'v':
				version();
				exit(0);
			default:
				break;
		}
	}

	if (optind < argc) {
		quickOpt = 1;
	} else if (optind >= argc && (quickOpt || phonOpt)) {
		fprintf(stderr, "Error: %s\n", getErrMsg("arg"));
		usage(argv[0]);
		exit(1);
	}

	for (; optind < argc; ++optind) {
		strcat(word, argv[optind]);
	}

	rcfile_load();

	if (dicName != NULL) {
		if (dic_list[0] != NULL) {
			free(dic_list[0]);
		}
		dic_list[0] = strdup(dicName);
	}

	if (quickOpt == 1 || phonOpt == 1) {
		allSearch(word);
	} else if (cmdOpt == 1 || dicName != NULL) {
		commandLine();
	} else {
		usage(argv[0]);
	}

	return 0;
}

==> ldic-0.3.0.tar.gz: ldic-0.3.0/src/ldic.c <==
/* L-Dic(Linux Dictionary)
 * Copyright (C) 2002 Baek SeungSoon.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <malloc.h>
#include <string.h>
#include <getopt.h>

#include <gdk/gdk.h>
#include <gtk/gtk.h>

#include "ldic.h"
#include "wlist.h"
#include "rcfile.h"
#include "readdic.h"
#include "fileops.h"
#include "tagtrans.h"

#include "window.h"

/* dicLoad:   б
 */
FILE
*dicLoad(char *dicName)
{
	FILE *fp;
	
	fp = fopen(dicName, "r");
	if (fp == NULL) {
		fprintf(stderr, "Error: %s, %s\n", dicName, getErrMsg("dic"));
		return NULL;
	}

	if (isDic(fp) == 0) {
		fprintf(stderr, "Error: %s, %s\n", dicName, getErrMsg("notdic"));
		return NULL;
	}
	
	return fp;
}

/* fileSearch: Ͽ ܾ ˻
 */
void
fileSearch(char *word)
{
	int i;
	
	for (i = 0; i < DICTIONARY_MAX; i++) {
		FILE *fp;

		if (dic_list[i] == NULL)
			continue;
	
		fp = dicLoad(dic_list[i]);
		if (fp == NULL) {
			continue;
		} else {
			long offset;
			char *s;
			
			printf("%s ˻..\n", dic_list[i]);

			offset = wordSearch(word, fp);
			if (offset == -1) {
				fclose(fp);
				continue;
			}
			
			s = offsetSearch(offset, fp);
			fclose(fp);
			
			printMean(s);

			free(s);
			break;
		}
	}
	
	if (i == DICTIONARY_MAX) {
		fprintf(stderr, "Error: %s\n", getErrMsg("word"));
	}
}

/* commandLine: Console ܾ  ˻
 */
void
commandLine(void)
{
	int i;
	WList *wlist = NULL;
	char word[BUFSIZ+1];
	FILE *fp;
	
	write(1, "Words List Loding...\t\t", 23);
	for (i = 0; i < DICTIONARY_MAX; i++) {
		if (dic_list[i] == NULL) {
			continue;
		}
		
		fp = dicLoad(dic_list[i]);
		if (fp == NULL) {
			continue;
		}
		
		wlist = memList(wlist, i, fp);
		fclose(fp);
	}
	printf("[Done]\n");

	for (;;) {
		WList *temp = NULL;
		
		write(1, "Search Word ('/exit'): ", 23);
		
		fgets(word, sizeof(word), stdin);
		word[strlen(word)-1] = '\0';

		if (word[0] == '/') {
			if(strcmp(word, "/exit") == '\0') {
				break;
			} else {
				printf("Error: %s\n", getErrMsg("cmd"));
			}
			continue;
		}

		temp = ListSearchWord(wlist, word);
		if (temp != NULL) {
			char *s;
			fp = dicLoad(dic_list[temp->num]);
			s = offsetSearch(temp->offset, fp);
			printMean(s);
			fclose(fp);
		} else {
			printf("Error: %s\n", getErrMsg("word"));
		}
	}

	freeall(wlist);
}

/* printMean: ܾ  
 */
void
printMean(char *s)
{
	int i;
	char **tagtext;
	
	tagtext = TagAndTextList(s);
	printf("[_mean_]\n");
	for (i = 0; tagtext[i] != NULL; i++) {
		if (tagtext[i][0] == '<' || (tagtext[i][0] == '&' && tagtext[i][1] != '[')) {
			if (strcmp(tagtext[i], "<br>") == '\0') {
				printf("\n");
			} else if (strncmp(tagtext[i], "&lt;", 4) == '\0') {
				printf("<");
			} else if (strncmp(tagtext[i], "&gt;", 4) == '\0') {
				printf(">");
			} else if (strncmp(tagtext[i], "&nbsp;", 4) == '\0') {
				printf(" ");
			} else if (strncmp(tagtext[i], "<img src", 8) == '\0') {
				printf("_");
			}
		} else {
			printf("%s", tagtext[i]);
		}
	}
	printf("\n");
}

/* getErrMsg: err شϴ ޼ ȯ
 */
char
*getErrMsg(char *err)
{
	static char retval[BUFSIZ+1];
	
	char *errList[] = {
		"arg",    "Էµ ܾ ϴ.",
		"list",   " ã  ϴ.",
		"dic",    " ã  ϴ.",
		"notdic", "   Դϴ.",
		"word",   "ܾ ã  ϴ.",
		"cmd",    "ӵ  ƴմϴ.",
		NULL
	};

	int i;

	for (i = 0; errList[i] != NULL; i += 2) {
		if (strcmp(err, errList[i]) == '\0') {
			strcpy(retval, errList[i+1]);
			break;
		}
	}

	if (errList[i] == NULL) {
		strcpy(retval, "    ߻߽ϴ.");
	}

	return retval;
}

/* usage: α׷  
 */
void
usage(const char *str)
{
	char *s;
	
	if ((s = strrchr(str, '/')) != NULL) {
		printf(": %s [ɼ]... [ܾ]\n", s+1);
	} else {
		printf(": %s [ɼ]... [ܾ]\n", str);
	}

	printf("\n");
	printf("  -c,     \tӰ˻\n");
	printf("  -q WORD,\t˻  \n");
	printf("  -d FILE,\t \n");
	printf("  -h,     \t   \n");
	printf("  -v,     \tα׷    \n");
	printf("\n");
	printf("<tadboy@ldic.kldp.net>    ּ\n");
}

/* version: α׷  
 */
void
version(void)
{
	printf("L-Dic(Linux Dictionary) Version %s\n", VERSION);
	printf("¼  ϴ.\n\n");
	printf("Copyright (C) 2002 Free Software Foundation, Inc.\n");
	printf(" α׷  ƮԴϴ. "
		"  ҽ Ͻʽÿ. "
		"ǰ̳ Ư   ռ Ͽ, "
		"   ʽϴ.\n");
}

/* main: α׷ ϰ, ɼ ó
 */
int
main(int argc, char *argv[])
{
	char c;
	char word[BUFSIZ+1];
	char *dicName = NULL;

	int cmdOpt = 0;		/* Ӱ˻ ɼ */
	int quickOpt = 0;	/* ˻ ɼ */
	int dbOpt = 0;		/*   */

	while ((c =  getopt(argc, argv,	"cqd:h?v")) != -1) {
		switch (c) {
			case 'c':
				cmdOpt = 1;
				break;
			case 'q':
				quickOpt = 1;
				break;
			case 'd':
				dbOpt = 1;
				dicName = (char *)malloc(strlen(optarg)+1);
				strcpy(dicName, optarg);
				break;
			case 'h':
			case '?':
				usage(argv[0]);
				exit(0);
			case 'v':
				version();
				exit(0);
			default:
				break;
		}
	}

	if (optind >= argc && quickOpt) {
		fprintf(stderr, "Error: %s\n", getErrMsg("arg"));
		usage(argv[0]);
		exit(1);
	} else if (optind < argc) {
		quickOpt = 1;
	}

	for (; optind < argc; ++optind) {
		strcat(word, argv[optind]);
	}

	check_for_home_path(LDIC_RC_DIR);
	
	load_options();

	if (dicName != NULL) {
		if (dic_list[0] != NULL) {
			free(dic_list[0]);
		}
		dic_list[0] = (char *)malloc(strlen(dicName)+1);
		strcpy(dic_list[0], dicName);
		free(dicName);
	}

	if (quickOpt == 1) {
		fileSearch(word);
	} else if (cmdOpt == 1 || dicName != NULL) {
		commandLine();
	} else {
		gtk_set_locale();
		gtk_init(&argc, &argv);
		
		create_window();
		diclist_refresh();
		wordlist_refresh();

		gtk_main();
	}

	return 0;
}
